@@ -20,14 +20,9 @@ from http.cookiejar import CookieJar
 from http.cookies import Morsel
 from urllib.parse import urlparse, urlunparse
 
-from ._compat import HAS_LEGACY_URLLIB3
+from .packages.urllib3 import BaseHTTPResponse
 from .utils import parse_scheme
 
-if HAS_LEGACY_URLLIB3 is False:
-    from urllib3 import BaseHTTPResponse
-else:
-    from urllib3_future import BaseHTTPResponse  # type: ignore[assignment]
-
 if typing.TYPE_CHECKING:
     from .models import PreparedRequest, Request
 
@@ -110,9 +105,7 @@ class MockRequest:
 
     def add_header(self, key, val):
         """cookiejar has no legitimate use for this method; add it back if you find one."""
-        raise NotImplementedError(
-            "Cookie headers should be added with add_unredirected_header()"
-        )
+        raise NotImplementedError("Cookie headers should be added with add_unredirected_header()")
 
     def add_unredirected_header(self, name, value):
         self._new_headers[name] = value
@@ -166,9 +159,7 @@ def extract_cookies_to_jar(
     :param response: urllib3.HTTPResponse object
     """
     if request is None or response is None:
-        raise ValueError(
-            "Attempt to extract cookie from undefined request and/or response"
-        )
+        raise ValueError("Attempt to extract cookie from undefined request and/or response")
 
     if not (hasattr(response, "_original_response") and response._original_response):
         return
@@ -233,9 +224,7 @@ class RequestsCookieJar(cookielib.CookieJar, MutableMapping):
     .. warning:: dictionary operations that are normally O(1) may be O(n).
     """
 
-    def __init__(
-        self, policy: cookielib.CookiePolicy | None = None, thread_free: bool = False
-    ):
+    def __init__(self, policy: cookielib.CookiePolicy | None = None, thread_free: bool = False):
         super().__init__(policy=policy or CookiePolicyLocalhostBypass())
         if thread_free:
             from .structures import DummyLock
@@ -261,9 +250,7 @@ class RequestsCookieJar(cookielib.CookieJar, MutableMapping):
         """
         # support client code that unsets cookies by assignment of a None value:
         if value is None:
-            remove_cookie_by_name(
-                self, name, domain=kwargs.get("domain"), path=kwargs.get("path")
-            )
+            remove_cookie_by_name(self, name, domain=kwargs.get("domain"), path=kwargs.get("path"))
             return
 
         if isinstance(value, Morsel):
@@ -353,18 +340,14 @@ class RequestsCookieJar(cookielib.CookieJar, MutableMapping):
             domains.append(cookie.domain)
         return False  # there is only one domain in jar
 
-    def get_dict(
-        self, domain: str | None = None, path: str | None = None
-    ) -> dict[str, str | None]:
+    def get_dict(self, domain: str | None = None, path: str | None = None) -> dict[str, str | None]:
         """Takes as an argument an optional domain and path and returns a plain
         old Python dict of name-value pairs of cookies that meet the
         requirements.
         """
         dictionary = {}
         for cookie in iter(self):
-            if (domain is None or cookie.domain == domain) and (
-                path is None or cookie.path == path
-            ):
+            if (domain is None or cookie.domain == domain) and (path is None or cookie.path == path):
                 dictionary[cookie.name] = cookie.value
         return dictionary
 
@@ -397,11 +380,7 @@ class RequestsCookieJar(cookielib.CookieJar, MutableMapping):
         remove_cookie_by_name(self, name)
 
     def set_cookie(self, cookie, *args, **kwargs):
-        if (
-            hasattr(cookie.value, "startswith")
-            and cookie.value.startswith('"')
-            and cookie.value.endswith('"')
-        ):
+        if hasattr(cookie.value, "startswith") and cookie.value.startswith('"') and cookie.value.endswith('"'):
             cookie.value = cookie.value.replace('\\"', "")
         return super().set_cookie(cookie, *args, **kwargs)
 
@@ -452,9 +431,7 @@ class RequestsCookieJar(cookielib.CookieJar, MutableMapping):
                     if path is None or cookie.path == path:
                         if toReturn is not None:
                             # if there are multiple cookies that meet passed in criteria
-                            raise CookieConflictError(
-                                f"There are multiple cookies with name, {name!r}"
-                            )
+                            raise CookieConflictError(f"There are multiple cookies with name, {name!r}")
                         # we will eventually return this as long as no cookie conflict
                         toReturn = cookie.value
 
@@ -526,9 +503,7 @@ def create_cookie(name, value, **kwargs):
 
     badargs = set(kwargs) - set(result)
     if badargs:
-        raise TypeError(
-            f"create_cookie() got unexpected keyword arguments: {list(badargs)}"
-        )
+        raise TypeError(f"create_cookie() got unexpected keyword arguments: {list(badargs)}")
 
     result.update(kwargs)
     result["port_specified"] = bool(result["port"])
