@@ -18,66 +18,44 @@ import struct
 import sys
 import tempfile
 import typing
-import wassima
 from collections import OrderedDict
 from functools import lru_cache
 from http.cookiejar import CookieJar
+from netrc import NetrcParseError, netrc
 from urllib.parse import quote, unquote, urlparse, urlunparse
-from urllib.request import (  # type: ignore[attr-defined]
+from urllib.request import (  # type: ignore[attr-defined]  # type: ignore[attr-defined]
     getproxies,
     getproxies_environment,
-)
-from urllib.request import parse_http_list as _parse_list_header
-from urllib.request import (  # type: ignore[attr-defined]
     proxy_bypass,
     proxy_bypass_environment,
 )
-from netrc import NetrcParseError, netrc
+from urllib.request import parse_http_list as _parse_list_header
 
-from ._compat import HAS_LEGACY_URLLIB3
-
-if HAS_LEGACY_URLLIB3 is False:
-    from urllib3.util import make_headers, parse_url
-    from urllib3.contrib.resolver import (
-        BaseResolver,
-        ProtocolResolver,
-        ResolverDescription,
-        ManyResolver,
-    )
-    from urllib3.contrib.resolver._async import (
-        AsyncBaseResolver,
-        AsyncResolverDescription,
-        AsyncManyResolver,
-    )
-    from urllib3 import ConnectionInfo
-    from urllib3.contrib.webextensions import load_extension, ExtensionFromHTTP
-    from urllib3.contrib.webextensions._async import AsyncExtensionFromHTTP
-else:
-    from urllib3_future.util import make_headers, parse_url  # type: ignore[assignment]
-    from urllib3_future.contrib.resolver import (  # type: ignore[assignment]
-        BaseResolver,
-        ProtocolResolver,
-        ResolverDescription,
-        ManyResolver,
-    )
-    from urllib3_future.contrib.resolver._async import (  # type: ignore[assignment]
-        AsyncBaseResolver,
-        AsyncResolverDescription,
-        AsyncManyResolver,
-    )
-    from urllib3_future import ConnectionInfo  # type: ignore[assignment]
-    from urllib3_future.contrib.webextensions import load_extension, ExtensionFromHTTP  # type: ignore[assignment]
-    from urllib3_future.contrib.webextensions._async import AsyncExtensionFromHTTP  # type: ignore[assignment]
+import wassima
 
 from .__version__ import __version__
-from .exceptions import InvalidURL, UnrewindableBodyError, MissingSchema
+from .exceptions import InvalidURL, MissingSchema, UnrewindableBodyError
+from .packages.urllib3 import ConnectionInfo
+from .packages.urllib3.contrib.resolver import (
+    BaseResolver,
+    ManyResolver,
+    ProtocolResolver,
+    ResolverDescription,
+)
+from .packages.urllib3.contrib.resolver._async import (
+    AsyncBaseResolver,
+    AsyncManyResolver,
+    AsyncResolverDescription,
+)
+from .packages.urllib3.contrib.webextensions import ExtensionFromHTTP, load_extension
+from .packages.urllib3.contrib.webextensions._async import AsyncExtensionFromHTTP
+from .packages.urllib3.util import make_headers, parse_url
 from .structures import CaseInsensitiveDict
 
 if typing.TYPE_CHECKING:
+    from ._typing import AsyncResolverType, ResolverType
     from .cookies import RequestsCookieJar
-    from .models import PreparedRequest, Request, Response
-    from ._typing import ResolverType, AsyncResolverType
-    from ._async import AsyncResponse
+    from .models import AsyncResponse, PreparedRequest, Request, Response
 
 
 getproxies = lru_cache()(getproxies)
@@ -88,9 +66,7 @@ NETRC_FILES = (".netrc", "_netrc")
 DEFAULT_PORTS: dict[str, int] = {"http": 80, "https": 443}
 
 # Ensure that ', ' is used to preserve previous delimiter behavior.
-DEFAULT_ACCEPT_ENCODING: str = ", ".join(
-    re.split(r",\s*", make_headers(accept_encoding=True)["accept-encoding"])
-)
+DEFAULT_ACCEPT_ENCODING: str = ", ".join(re.split(r",\s*", make_headers(accept_encoding=True)["accept-encoding"]))
 
 
 if sys.platform == "win32":
@@ -199,28 +175,18 @@ def super_len(o: typing.Any) -> int:
 
 
 @lru_cache(maxsize=64)
-def get_netrc_auth(
-    url: str | None, raise_errors: bool = False
-) -> tuple[str, str] | None:
+def get_netrc_auth(url: str | None, raise_errors: bool = False) -> tuple[str, str] | None:
     """Returns the Requests tuple auth for a given url from netrc."""
 
     if url is None:
         return None
 
     netrc_file = os.environ.get("NETRC")
-    netrc_locations: tuple[str, ...] = (
-        (netrc_file,)
-        if netrc_file is not None
-        else tuple(f"~/{f}" for f in NETRC_FILES)
-    )
+    netrc_locations: tuple[str, ...] = (netrc_file,) if netrc_file is not None else tuple(f"~/{f}" for f in NETRC_FILES)
 
     try:
         netrc_path = next(
-            (
-                os.path.expanduser(f)
-                for f in netrc_locations
-                if os.path.exists(os.path.expanduser(f))
-            ),
+            (os.path.expanduser(f) for f in netrc_locations if os.path.exists(os.path.expanduser(f))),
             None,
         )
         if netrc_path is None:
@@ -247,9 +213,7 @@ def get_netrc_auth(
             raise OSError("Problem accessing or reading the netrc file") from e
     except AttributeError as e:
         if raise_errors:
-            raise RuntimeError(
-                "Unexpected error while retrieving netrc authenticators"
-            ) from e
+            raise RuntimeError("Unexpected error while retrieving netrc authenticators") from e
     return None
 
 
@@ -434,9 +398,7 @@ def dict_from_cookiejar(cj: CookieJar) -> dict[str, str | None]:
     return cookie_dict
 
 
-def add_dict_to_cookiejar(
-    cj: RequestsCookieJar, cookie_dict
-) -> RequestsCookieJar | CookieJar:
+def add_dict_to_cookiejar(cj: RequestsCookieJar, cookie_dict) -> RequestsCookieJar | CookieJar:
     """Returns a CookieJar from a key/value dictionary.
 
     :param cj: CookieJar to insert cookies into.
@@ -490,11 +452,7 @@ def get_encoding_from_headers(headers: typing.Mapping[str, str]) -> str | None:
     content_type, params = _parse_content_type_header(content_type)
 
     if "charset" in params:
-        return (
-            params["charset"].strip("'\"")
-            if isinstance(params["charset"], str)
-            else None
-        )
+        return params["charset"].strip("'\"") if isinstance(params["charset"], str) else None
 
     if "application/json" in content_type:
         # Assume UTF-8 based on RFC 4627: https://www.ietf.org/rfc/rfc4627.txt since the charset was unset
@@ -546,9 +504,7 @@ async def astream_decode_response_unicode(
 _SV = typing.TypeVar("_SV", str, bytes)
 
 
-def iter_slices(
-    string: _SV, slice_length: int | None
-) -> typing.Generator[_SV, None, None]:
+def iter_slices(string: _SV, slice_length: int | None) -> typing.Generator[_SV, None, None]:
     """Iterate over slices of a string."""
     pos = 0
     if slice_length is None or slice_length <= 0:
@@ -559,9 +515,7 @@ def iter_slices(
 
 
 # The unreserved URI characters (RFC 3986)
-UNRESERVED_SET = frozenset(
-    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz" + "0123456789-._~"
-)
+UNRESERVED_SET = frozenset("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz" + "0123456789-._~")
 
 
 def unquote_unreserved(uri: str) -> str:
@@ -631,14 +585,10 @@ def address_in_network(ip: str, net: str) -> bool:
         netmask = _get_mask_bits(int(bits))
         network = struct.unpack(">L", socket.inet_aton(netaddr))[0]
     elif is_ipv6_address(ip) and is_ipv6_address(netaddr):
-        ipaddr_msb, ipaddr_lsb = struct.unpack(
-            ">QQ", socket.inet_pton(socket.AF_INET6, ip)
-        )
+        ipaddr_msb, ipaddr_lsb = struct.unpack(">QQ", socket.inet_pton(socket.AF_INET6, ip))
         ipaddr = (ipaddr_msb << 64) ^ ipaddr_lsb
         netmask = _get_mask_bits(int(bits), 128)
-        network_msb, network_lsb = struct.unpack(
-            ">QQ", socket.inet_pton(socket.AF_INET6, netaddr)
-        )
+        network_msb, network_lsb = struct.unpack(">QQ", socket.inet_pton(socket.AF_INET6, netaddr))
         network = (network_msb << 64) ^ network_lsb
     else:
         return False
@@ -675,9 +625,7 @@ def compare_ipv6(a: str, b: str):
     Compare 2 IPs, uses socket.inet_pton to normalize IPv6 IPs
     """
     try:
-        return socket.inet_pton(socket.AF_INET6, a) == socket.inet_pton(
-            socket.AF_INET6, b
-        )
+        return socket.inet_pton(socket.AF_INET6, a) == socket.inet_pton(socket.AF_INET6, b)
     except OSError:
         return False
 
@@ -753,9 +701,7 @@ def should_bypass_proxies(url: str, no_proxy: str | None) -> bool:
     if no_proxy:
         # We need to check whether we match here. We need to see if we match
         # the end of the hostname, both with and without the port.
-        no_proxy_list = list(
-            host for host in no_proxy.replace(" ", "").split(",") if host
-        )
+        no_proxy_list = list(host for host in no_proxy.replace(" ", "").split(",") if host)
 
         if is_ipv4_address(parsed.hostname) or is_ipv6_address(parsed.hostname):
             for proxy_ip in no_proxy_list:
@@ -807,9 +753,7 @@ def get_environ_proxies(url: str, no_proxy: str | None = None) -> dict[str, str]
         return proxies
 
 
-def select_proxy(
-    url: str, proxies: typing.Mapping[str, str | None] | None
-) -> str | None:
+def select_proxy(url: str, proxies: typing.Mapping[str, str | None] | None) -> str | None:
     """Select a proxy for the url, if applicable.
 
     :param url: The url being for the request
@@ -839,18 +783,14 @@ def select_proxy(
         implementation = None
 
         if "+" in maybe_extension_scheme:
-            maybe_extension_scheme, implementation = tuple(
-                maybe_extension_scheme.split("+", maxsplit=1)
-            )
+            maybe_extension_scheme, implementation = tuple(maybe_extension_scheme.split("+", maxsplit=1))
 
         try:
             extension_class = load_extension(maybe_extension_scheme, implementation)
         except ImportError:
             pass
         else:
-            parent_scheme = extension_class.scheme_to_http_scheme(
-                maybe_extension_scheme
-            )
+            parent_scheme = extension_class.scheme_to_http_scheme(maybe_extension_scheme)
 
             proxy_keys.append(parent_scheme)
             proxy_keys.append(parent_scheme + "://" + urlparts.hostname)
@@ -1019,9 +959,7 @@ def rewind_body(prepared_request: PreparedRequest) -> None:
         try:
             body_seek(prepared_request._body_position)
         except OSError:
-            raise UnrewindableBodyError(
-                "An error occurred when rewinding request body for redirect."
-            )
+            raise UnrewindableBodyError("An error occurred when rewinding request body for redirect.")
     else:
         raise UnrewindableBodyError("Unable to rewind request body for redirect.")
 
@@ -1079,18 +1017,12 @@ def create_resolver(definition: ResolverType | None) -> BaseResolver:
             can_resolve_localhost = True
 
     if not can_resolve_localhost:
-        resolvers.append(
-            ResolverDescription.from_url("system://default?hosts=localhost")
-        )
+        resolvers.append(ResolverDescription.from_url("system://default?hosts=localhost"))
 
     #: We want to automatically forward ca_cert_data, ca_cert_dir, and ca_certs.
     for rd in resolvers:
         # If no CA bundle is provided, inject the system's default!
-        if (
-            "ca_cert_data" not in rd
-            and "ca_cert_dir" not in rd
-            and "ca_certs" not in rd
-        ):
+        if "ca_cert_data" not in rd and "ca_cert_dir" not in rd and "ca_certs" not in rd:
             rd["ca_cert_data"] = wassima.generate_ca_bundle()
 
     return ManyResolver(*[r.new() for r in resolvers])
@@ -1149,26 +1081,18 @@ def create_async_resolver(definition: AsyncResolverType | None) -> AsyncBaseReso
             can_resolve_localhost = True
 
     if not can_resolve_localhost:
-        resolvers.append(
-            AsyncResolverDescription.from_url("system://default?hosts=localhost")
-        )
+        resolvers.append(AsyncResolverDescription.from_url("system://default?hosts=localhost"))
 
     #: We want to automatically forward ca_cert_data, ca_cert_dir, and ca_certs.
     for rd in resolvers:
         # If no CA bundle is provided, inject the system's default!
-        if (
-            "ca_cert_data" not in rd
-            and "ca_cert_dir" not in rd
-            and "ca_certs" not in rd
-        ):
+        if "ca_cert_data" not in rd and "ca_cert_dir" not in rd and "ca_certs" not in rd:
             rd["ca_cert_data"] = wassima.generate_ca_bundle()
 
     return AsyncManyResolver(*[r.new() for r in resolvers])
 
 
-def resolve_socket_family(
-    disable_ipv4: bool, disable_ipv6: bool
-) -> socket.AddressFamily:
+def resolve_socket_family(disable_ipv4: bool, disable_ipv6: bool) -> socket.AddressFamily:
     if disable_ipv4:
         return socket.AF_INET6
     if disable_ipv6:
@@ -1179,9 +1103,7 @@ def resolve_socket_family(
 def _swap_context(response: AsyncResponse | Response) -> None:
     response_class = response.__class__
 
-    is_async = (
-        len(response_class.__bases__) == 1 and response_class.__bases__[0] is not object
-    )
+    is_async = len(response_class.__bases__) == 1 and response_class.__bases__[0] is not object
 
     if is_async:
         response.__class__ = response_class.__bases__[0]
@@ -1201,7 +1123,7 @@ def _deepcopy_ci(o: ConnectionInfo | None) -> ConnectionInfo | None:
     return n
 
 
-def parse_scheme(url: str, default: str | None = None, max_length: int = 9) -> str:
+def parse_scheme(url: str, default: str | None = None, max_length: int = 11) -> str:
     """We tend to extract url scheme often enough, but we were crazy
     enough to use urlparse for it...! We were wasting precious CPU cycles.
     Return used scheme url, lowercased."""
@@ -1210,21 +1132,14 @@ def parse_scheme(url: str, default: str | None = None, max_length: int = 9) -> s
     except ValueError as e:
         if default is not None:
             return default
-        raise MissingSchema(
-            f"Invalid URL {url!r}: No scheme supplied. "
-            f"Perhaps you meant https://{url}?"
-        ) from e
+        raise MissingSchema(f"Invalid URL {url!r}: No scheme supplied. Perhaps you meant https://{url}?") from e
 
     return scheme.lower()
 
 
 def is_ocsp_capable(conn_info: ConnectionInfo | None) -> bool:
     # we can't do anything in that case.
-    if (
-        conn_info is None
-        or conn_info.certificate_der is None
-        or conn_info.certificate_dict is None
-    ):
+    if conn_info is None or conn_info.certificate_der is None or conn_info.certificate_dict is None:
         return False
 
     endpoints: list[str] = [  # type: ignore
@@ -1241,6 +1156,24 @@ def is_ocsp_capable(conn_info: ConnectionInfo | None) -> bool:
     return True
 
 
+def is_crl_capable(conn_info: ConnectionInfo | None) -> bool:
+    if conn_info is None or conn_info.certificate_der is None or conn_info.certificate_dict is None:
+        return False
+
+    endpoints: list[str] = [  # type: ignore
+        # exclude non-HTTP endpoint. like ldap.
+        ep  # type: ignore
+        for ep in list(conn_info.certificate_dict.get("crlDistributionPoints", []))  # type: ignore
+        if ep.startswith("http://")  # type: ignore
+    ]
+
+    # well... not all issued certificate have a CRL distribution endpoint
+    if not endpoints:
+        return False
+
+    return True
+
+
 def wrap_extension_for_http(
     extension: type[ExtensionFromHTTP],
 ) -> type[ExtensionFromHTTP]:
@@ -1249,43 +1182,41 @@ def wrap_extension_for_http(
     This function purposely wrap the extension class to achieve that.
     Warning: synchronous context only!
     """
-    if HAS_LEGACY_URLLIB3 is False:
-        from urllib3.exceptions import (
-            ClosedPoolError,
-            HTTPError as _HTTPError,
-            InvalidHeader as _InvalidHeader,
-            ProtocolError,
-            ProxyError as _ProxyError,
-            ReadTimeoutError,
-            SSLError as _SSLError,
-            DecodeError,
-        )
-    else:
-        from urllib3_future.exceptions import (  # type: ignore[assignment]
-            ClosedPoolError,
-            HTTPError as _HTTPError,
-            InvalidHeader as _InvalidHeader,
-            ProtocolError,
-            ProxyError as _ProxyError,
-            ReadTimeoutError,
-            SSLError as _SSLError,
-            DecodeError,
-        )
 
     from .exceptions import (
+        ChunkedEncodingError,
         ConnectionError,
+        ContentDecodingError,
         InvalidHeader,
         ProxyError,
         ReadTimeout,
+    )
+    from .exceptions import (
         SSLError as RequestsSSLError,
-        ContentDecodingError,
-        ChunkedEncodingError,
+    )
+    from .packages.urllib3.exceptions import (
+        ClosedPoolError,
+        DecodeError,
+        ProtocolError,
+        ReadTimeoutError,
+    )
+    from .packages.urllib3.exceptions import (
+        HTTPError as _HTTPError,
+    )
+    from .packages.urllib3.exceptions import (
+        InvalidHeader as _InvalidHeader,
+    )
+    from .packages.urllib3.exceptions import (
+        ProxyError as _ProxyError,
+    )
+    from .packages.urllib3.exceptions import (
+        SSLError as _SSLError,
     )
 
     class _WrappedExtensionFromHTTP(extension):  # type: ignore[valid-type,misc]
-        def next_payload(self) -> str | bytes | None:
+        def next_payload(self, *args, **kwargs) -> str | bytes | None:
             try:
-                return super().next_payload()
+                return super().next_payload(*args, **kwargs)
             except ProtocolError as e:
                 raise ChunkedEncodingError(e)
             except DecodeError as e:
@@ -1344,43 +1275,41 @@ def async_wrap_extension_for_http(
     This function purposely wrap the extension class to achieve that.
     Warning: asynchronous context only!
     """
-    if HAS_LEGACY_URLLIB3 is False:
-        from urllib3.exceptions import (
-            ClosedPoolError,
-            HTTPError as _HTTPError,
-            InvalidHeader as _InvalidHeader,
-            ProtocolError,
-            ProxyError as _ProxyError,
-            ReadTimeoutError,
-            SSLError as _SSLError,
-            DecodeError,
-        )
-    else:
-        from urllib3_future.exceptions import (  # type: ignore[assignment]
-            ClosedPoolError,
-            HTTPError as _HTTPError,
-            InvalidHeader as _InvalidHeader,
-            ProtocolError,
-            ProxyError as _ProxyError,
-            ReadTimeoutError,
-            SSLError as _SSLError,
-            DecodeError,
-        )
 
     from .exceptions import (
+        ChunkedEncodingError,
         ConnectionError,
+        ContentDecodingError,
         InvalidHeader,
         ProxyError,
         ReadTimeout,
+    )
+    from .exceptions import (
         SSLError as RequestsSSLError,
-        ContentDecodingError,
-        ChunkedEncodingError,
+    )
+    from .packages.urllib3.exceptions import (
+        ClosedPoolError,
+        DecodeError,
+        ProtocolError,
+        ReadTimeoutError,
+    )
+    from .packages.urllib3.exceptions import (
+        HTTPError as _HTTPError,
+    )
+    from .packages.urllib3.exceptions import (
+        InvalidHeader as _InvalidHeader,
+    )
+    from .packages.urllib3.exceptions import (
+        ProxyError as _ProxyError,
+    )
+    from .packages.urllib3.exceptions import (
+        SSLError as _SSLError,
     )
 
     class _AsyncWrappedExtensionFromHTTP(extension):  # type: ignore[valid-type,misc]
-        async def next_payload(self) -> str | bytes | None:
+        async def next_payload(self, *args, **kwargs) -> str | bytes | None:
             try:
-                return await super().next_payload()
+                return await super().next_payload(*args, **kwargs)
             except ProtocolError as e:
                 raise ChunkedEncodingError(e)
             except DecodeError as e:
